/*
 * Model.java
 * Created on Aug 14, 2011
 */
package com.areaofthoughts.eZoo;

import java.io.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
/**
 * Model
 * @author Mark
 */
class Model {
    
    private ArrayList<String> adjectives = null;
    private Random randObj = null;
    private static final int MAX_ANIMALS = 6;
    private static final int BOM_SIZE = 3; //UTF-8
    /**
     * Constructor
     */
    Model() {
        adjectives = new ArrayList<String>();
        randObj = new Random();
    }
    /**
     * Animal id enumerator.
     */
    private enum Animal
    {
        TURTLE { 
            @Override
            public String toString(){ return "Turtle"; }
        }, PANDA {
            @Override
            public String toString(){ return "Panda"; }
        }, DOLPHIN {
            @Override
            public String toString(){ return "Dolphin"; }
        }, ELEPHANT {
            @Override
            public String toString(){ return "Elephant"; }
        }, BUTTERFLY {
            @Override
            public String toString(){ return "Butterfly"; }
        }, HIPPOPOTAMUS {
            @Override
            public String toString(){ return "Hippopotamus"; }
        };
    }
    /**
     * Fills adjective array list with values from a UTF-8 text file.<br />
     * <br />
     * A UTF-8 stream can optionally begin with a byte order mark (see, 
     * for example http://www.unicode.org/unicode/faq/utf_bom.html).  
     * This is the character FEFF, which is represented as EF BB BF in 
     * UTF-8. Java's UTF-8 encoding does not recognize this character 
     * as a BOM, though; the result of reading such a stream is a set of 
     * characters beginning with FEFF.<br />
     * <br />
     * Application code must recognize and skip the BOM itself.<br />
     * <br />
     * http://bugs.sun.com/view_bug.do?bug_id=4508058
     */
    void fillAdjArr() {
        String ln;
        byte[] b = new byte[BOM_SIZE];
        InputStream is = null;
        PushbackInputStream pbis = null;
        BufferedReader br = null;
        try {
            is = getClass().getResourceAsStream("/text/food_adjectives_UTF-8.txt");
            pbis = new PushbackInputStream(is, BOM_SIZE);
            pbis.read(b, 0, BOM_SIZE);
            // <editor-fold defaultstate="collapsed" desc="Unused Block">
            /**
            for(int i = 0; i < b.length; i++)
                System.out.format("%x ", b[i]);
            System.out.println();
            System.out.format("0xffffffef: %d 0xffffffff: %d%n", (byte)0xffffffef, (byte)0xffffffff);
            */// </editor-fold>
            if(b[0] != (byte)0xef && b[1] != (byte)0xbb && b[2] != (byte)0xbf)
                pbis.unread(b);
            br = new BufferedReader(new InputStreamReader(pbis, "UTF-8"));
            //System.out.println("File has UTF-8 BOM."); 
            Pattern pattern = Pattern.compile("^(\\W//)|^(//)|^(\\W#)|^#");
            while ((ln = br.readLine()) != null) {
                Matcher matcher = pattern.matcher(ln.toLowerCase().trim());
                if(!matcher.find()) { //Ignore lines with single line comment marks // or #.
                    //System.out.println(ln);
                    adjectives.add(ln);
                }
                else
                    continue;
            }
        } catch(IOException e) {
            System.err.println(e);
        } finally {
            close(br);
            close(pbis);
            close(is);
        }
   }
    /**
     * Method to close streams.
     * @param closable 
     * @return 
     */
    private void close(Closeable closable) {
        if (closable != null) {
            try {
                closable.close();
            } catch (IOException e) {
                System.err.println(e);
            }
        }
    }
    /**
     * Builds partial string for caption.
     * Uses regular expression to detect if first letter in input stream is a vowel.
     * If first letter is a vowel, prepend "An" to string, otherwise, prepend "A"
     * to string.
     * @return
     */
    String getPreString() {
        String element = adjectives.get(randObj.nextInt(adjectives.size()));
        //String element = adjectives.get(0);
        Pattern pattern = Pattern.compile("^[^aeiou]"); // The string starts with a consonant.
        Matcher matcher = pattern.matcher(element);
        String firstword;
        if(matcher.find())
            firstword = "A";
        else
            firstword = "An";
        return firstword + " " + element;
    }
    /**
     * Builds final caption string and returns to controller.
     * @param ac
     * @return
     */
    String getCaption(String ac) {
        String idstr;
        idstr = ac;
        return getPreString() + " " + idstr + ".";
    } 
    /**
     * Builds file path string for resource images.
     * @param ac
     * @return
     */
     String getImageFilePath(String ac) {
        String idstr;
        idstr = ac;
        return "/images/" + idstr + "/" + idstr + "_256x256.png";  // NOI18N
    }
    /**
     * Builds file path string for title bar image.
     * @return 
     */
    static String getAppImagePath(){
        return "/images/eZoo/eZoo_32x32.png";  // NOI18N
    }
    /**
     * Retrieves a random animal id string.
     * @return 
     */
    String getRandAnimal() {
        int ano = randObj.nextInt(MAX_ANIMALS);
        //Animal[] values = Animal.values();
        //System.out.println(Animal.values()[ano].toString());
        return Animal.values()[ano].toString();        
    }
    
    
    // <editor-fold defaultstate="collapsed" desc="Unused Method">
    /**
     * Tests output to console.
     * @param ac 
     */
    void printToConsole(String ac) {
        System.out.println(getPreString() + " " + ac + ".");
    }// </editor-fold>
    
    // <editor-fold defaultstate="collapsed" desc="Unused Method">
    /**
     * Tests output to file.
     */
    void printToFile(){
        BufferedWriter bw = null;
        File ofile = new File("outFile.txt"); // NOI18N
        Iterator itr = adjectives.iterator();
	try{
            bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(ofile), "UTF-8"));
            
            while (itr.hasNext()) {
                //System.out.println(itr.next());
		bw.write(itr.next().toString());
		if(itr.hasNext())
                    bw.newLine();
            }
        }catch (IOException e){
            System.err.println(e);
        } finally {
            close(bw);
        }
    }// </editor-fold>
}    

